import pandas as pd


class Constants:
    min_prediction = 0
    max_prediction = 4
    torch_seed = 1
    # hyperparameters
    batch_size_divider = 6
    n_epochs = 30
    gru_hidden_size = 200  # Could be a multiple of the prediction length (e.g.: 12 questions)
    gru_n_layers = 5
    ffnn_hidden_size = 27
    learning_rate = 0.01
    dropout = 0.12

    @classmethod
    def apply_best_param_config(cls, csv_file):
        # Read the CSV file into a DataFrame
        df = pd.read_csv(csv_file)

        # Update the class attributes with the parameters in the DataFrame
        for column in df.columns:
            if hasattr(cls, column):
                setattr(cls, column, df.loc[0, column])

        cls.hidden_size = int(cls.hidden_size)
        cls.n_epochs = int(cls.n_epochs)
        cls.n_layers = int(cls.n_layers)
        cls.learning_rate = float(cls.learning_rate)
        cls.should_clip_gradient = bool(cls.should_clip_gradient)
        cls.dropout = float(cls.dropout)

        print("Number of epochs: ", cls.n_epochs)
        print("Hidden size: %d" % cls.hidden_size)
        print("Number of layers: %d" % cls.n_layers)
        print("Learning rate: %f" % cls.learning_rate)
        print("Clipping gradient: %f" % cls.should_clip_gradient)
        print("Dropout: %f" % cls.dropout)