import torch
import torch.nn as nn
import pandas as pd

class Constants:
    min_prediction = 0
    max_prediction = 4
    torch_seed = 1
    # hyperparameters
    n_epochs = 45
    hidden_size = 12  # Could be a multiple of the prediction length (e.g.: k * 12 questions)
    n_layers = 1
    learning_rate = 0.01

    @classmethod
    def apply_best_param_config(cls, csv_file):
        df = pd.read_csv(csv_file)

        # Update the class attributes with the parameters in the DataFrame
        for column in df.columns:
            if hasattr(cls, column):
                setattr(cls, column, df.loc[0, column])

        cls.hidden_size = int(cls.hidden_size)
        cls.n_epochs = int(cls.n_epochs)
        cls.n_layers = int(cls.n_layers)
        cls.learning_rate = int(cls.learning_rate)


class SimpleLSTM(nn.Module):
    def __init__(self, n_layers, hidden_size, input_size, output_size):
        super(SimpleLSTM, self).__init__()

        self.hidden_dim = hidden_size
        self.lstm = nn.LSTM(input_size=input_size,
                            hidden_size=hidden_size,
                            num_layers=n_layers,
                            batch_first=True)

        self.fc = nn.Linear(hidden_size, output_size)
        self.relu = nn.ReLU()
        self.h0 = nn.Parameter(torch.zeros(n_layers, 1, hidden_size))
        self.c0 = nn.Parameter(torch.zeros(n_layers, 1, hidden_size))

    def forward(self, x):
        # Use learned initial states
        h0 = self.h0.repeat(1, x.size(0), 1).to(x.device)
        c0 = self.c0.repeat(1, x.size(0), 1).to(x.device)

        out, _ = self.lstm(x, (h0, c0))
        out = self.fc(out)
        out = self.relu(out)
        return out

    def mae_loss(self, y_pred, y_true):
        return torch.mean(torch.abs(y_pred - y_true))

    def rmse_loss(self, y_pred, y_true):
        mse = torch.mean((y_pred - y_true) ** 2)
        return torch.sqrt(mse)
