import torch
from torch import nn
import pandas as pd


class Constants:
    min_prediction = 0
    max_prediction = 4
    torch_seed = 1
    # hyperparameters
    should_clip_gradient = False
    n_epochs = 210
    hidden_size = 12  # Could be a multiple of the prediction length (e.g.: 12 questions)
    n_layers = 3
    learning_rate = 0.01
    dropout = 0.0

    @classmethod
    def apply_best_param_config(cls, csv_file):
        # Read the CSV file into a DataFrame
        df = pd.read_csv(csv_file)

        # Update the class attributes with the parameters in the DataFrame
        for column in df.columns:
            if hasattr(cls, column):
                setattr(cls, column, df.loc[0, column])

        cls.hidden_size = int(cls.hidden_size)
        cls.n_epochs = int(cls.n_epochs)
        cls.n_layers = int(cls.n_layers)
        cls.learning_rate = float(cls.learning_rate)
        cls.should_clip_gradient = bool(cls.should_clip_gradient)
        cls.dropout = float(cls.dropout)

        print("Number of epochs: ", cls.n_epochs)
        print("Hidden size: %d" % cls.hidden_size)
        print("Number of layers: %d" % cls.n_layers)
        print("Learning rate: %f" % cls.learning_rate)
        print("Clipping gradient: %f" % cls.should_clip_gradient)
        print("Dropout: %f" % cls.dropout)


class SimpleRNN(nn.Module):
    def __init__(self, n_layers, hidden_size, input_size, prediction_length, dropout):
        super(SimpleRNN, self).__init__()
        self.rnn = nn.RNN(num_layers=n_layers,
                          input_size=input_size,
                          hidden_size=hidden_size,
                          batch_first=True,
                          dropout=dropout)
        self.fc = nn.Linear(hidden_size, prediction_length)

    def forward(self, x):
        out, _ = self.rnn(x)
        out = self.fc(out)
        return out

    def mae_loss(self, y_pred, y_true):
        return torch.mean(torch.abs(y_pred - y_true))

    def rmse_loss(self, y_pred, y_true):
        mse = torch.mean((y_pred - y_true) ** 2)
        return torch.sqrt(mse)

    def mean_mae_rmse_loss(self, y_pred, y_true):
        mae_loss = self.mae_loss(y_pred, y_true)
        rmse_loss = self.rmse_loss(y_pred, y_true)
        return (mae_loss + rmse_loss) / 2
