import torch
from torch.nn.utils.rnn import pad_sequence
import pandas as pd
from RNNUtils import Constants, SimpleRNN

if __name__ == "__main__":
    torch.backends.cudnn.deterministic = True
    torch.backends.cudnn.benchmark = False

    dataWithNans = "train_all_first_last_nans.csv"

    train_df = pd.read_csv(f'../../{dataWithNans}')

    if train_df.isna().any().any():
        print("There are NaN values in the DataFrame. Applying forward fill.")
        train_df.ffill(inplace=True)
        if train_df.isna().any().any():
            print("There are still NaN values in the DataFrame after forward fill. Applying backwards fill.")
            train_df.bfill(inplace=True)
        else:
            print("There are no NaN values in the DataFrame after forward filling.")
    else:
        print("There are no NaN values in the DataFrame.")

    # Group the data by patient_id
    grouped_train = train_df.groupby('patient_id')

    # Convert each group to a list and store in a list
    X_train = [group.drop(
        ['patient_id', 'init_q', 'target_q', 'tQ1', 'tQ2', 'tQ3', 'tQ4', 'tQ5', 'tQ6', 'tQ7', 'tQ8', 'tQ9', 'tQ10',
         'tQ11',
         'tQ12'], axis=1).values.tolist() for _, group in grouped_train]
    y_train = [
        group[['tQ1', 'tQ2', 'tQ3', 'tQ4', 'tQ5', 'tQ6', 'tQ7', 'tQ8', 'tQ9', 'tQ10', 'tQ11', 'tQ12']].values.tolist()
        for
        _, group in grouped_train]

    # Convert lists to tensors
    X_train_tensors = [torch.tensor(x) for x in X_train]
    y_train_tensors = [torch.tensor(y) for y in y_train]

    # Pad the sequences
    X_train = pad_sequence(X_train_tensors, batch_first=True)
    y_train = pad_sequence(y_train_tensors, batch_first=True)

    n_features = X_train.shape[-1]
    prediction_length = y_train.shape[-1]

    model = SimpleRNN(n_layers=Constants.n_layers,
                      hidden_size=Constants.hidden_size,
                      input_size=n_features,
                      prediction_length=prediction_length)

    optimizer = torch.optim.Adam(model.parameters(), lr=Constants.lr)

    # Training
    for epoch in range(Constants.n_epochs):
        model.zero_grad()
        output = model(X_train)
        loss = model.mae_loss(output, y_train)
        loss.backward()

        # Gradient clipping
        torch.nn.utils.clip_grad_norm_(model.parameters(), max_norm=1)

        optimizer.step()
        if epoch % 10 == 0:
            print(f'Epoch: {epoch + 10}, Loss: {loss.item()}')

    # Evaluation
    model.eval()

    # with torch.no_grad():
    #     predictions = model(X_test)
    #     predictions = torch.round(predictions)
    #     predictions_clamped = torch.clamp(predictions,
    #                                       min=Constants.min_prediction,
    #                                       max=Constants.max_prediction)
    #
    #     test_loss = criterion(predictions, y_test)
    #     print(f'Test Loss: {test_loss.item()} \n')
