#%%
import pandas as pd
from GeneralUtils import fillNansBasic, drop_high_zero_null_columns_greater_than_threshold, drop_columns
import os


def extract_suffix_from_filename(filename):
    # Split the filename by the underscore character
    split_by_underscore = filename.split("_")

    # The desired string is the second, third, and fourth elements of the split list
    desired_string = "_".join(split_by_underscore[2:])

    # Split the desired string by the dot character to remove the file extension
    split_by_dot = desired_string.split(".")

    # The final string is the first element of this split list
    final_string = split_by_dot[0]

    return final_string


SHOULD_DROP_FIRST_LAST_TREND_FROM_STATIC = True
SHOULD_DROP_HIGH_ZERO_COLUMNS = False  # set to true to drop cols higher than threshold
DROP_COLUMNS_THRESHOLD = 95  # columns with greater 0/NaN percentage will be dropped
SPLIT_SEED = 4
TEST_RATIO = 0.3

FEATURE_COLUMNS_TO_DROP = ['init_q', 'target_q', 'tQ1', 'tQ2', 'tQ3', 'tQ4', 'tQ5', 'tQ6',
                           'tQ7', 'tQ8', 'tQ9', 'tQ10', 'tQ11', 'tQ12']
TARGET_COLUMNS = ['patient_id', 'tQ1', 'tQ2', 'tQ3', 'tQ4', 'tQ5', 'tQ6', 'tQ7', 'tQ8', 'tQ9', 'tQ10', 'tQ11', 'tQ12']

firstLastNans = "train_all_first_last_nans.csv"
firstLastNansDropped = "train_all_first_last_nans_dropped.csv"
firstLastNansInterpolatedSPODropped = "train_all_first_last_nans_interpolated_spo_dropped.csv"
firstLastNansInterpolatedTrend3 = "train_all_first_last_nans_interpolated_trend3_strip87.csv"
firstLastNansInterpolatedTrend5 = "train_all_first_last_nans_interpolated_trend5_strip87.csv"
NONCONSECfirstLastNansInterpolatedSPODropped = "train_all_nonconsec_first_last_nans_interpolated_spo_dropped_strip87.csv"
trainStatic = "train-static.csv"

preprocessedDataFileName = trainStatic
folder_name = extract_suffix_from_filename(preprocessedDataFileName)
sensor_file_name = "../train-sensor_interpolated_strip87.csv"
ground_truth_train_file = "../val.csv"

if SHOULD_DROP_FIRST_LAST_TREND_FROM_STATIC:
    FOLDER_NAME_SUFFIX = "_DROP_FIRST_LAST_TREND_FROM_STATIC"
    SPLIT_FILE_SUFFIX = "_STATIC"
else:
    FOLDER_NAME_SUFFIX = ""
    SPLIT_FILE_SUFFIX = ""

DEV_DATA_DIR = f'../dev_data/{folder_name}{FOLDER_NAME_SUFFIX}'

if __name__ == "__main__":

    df = pd.read_csv(f'../{preprocessedDataFileName}')
    df = fillNansBasic(df)

    if SHOULD_DROP_FIRST_LAST_TREND_FROM_STATIC:
        df = drop_columns(df, first=True, last=True, trend=True)
        sensor_data = pd.read_csv(sensor_file_name)
        sensor_data = fillNansBasic(sensor_data)
    else:
        sensor_data = None

    if SHOULD_DROP_HIGH_ZERO_COLUMNS:
        df = drop_high_zero_null_columns_greater_than_threshold(df, 95)

    # Count the number of entries for each patient
    patient_counts = df['patient_id'].value_counts().sort_values(ascending=True)

    sorted_patient_ids = patient_counts.index.tolist()
    num_test_patients = int(TEST_RATIO * len(sorted_patient_ids))

    test_ids = sorted_patient_ids[:num_test_patients]
    train_ids = sorted_patient_ids[num_test_patients:]

    train_df = df[df['patient_id'].isin(train_ids)]
    test_df = df[df['patient_id'].isin(test_ids)].groupby('patient_id').first().reset_index()

    print("Number of train patients:", len(train_ids))
    print("Number of test patients:", len(test_ids))

    print("Entries(rows) in train: ", len(train_df))
    print("Entries(rows) in test: ", len(test_df))

    print("Columns in train: ", len(train_df.columns))
    print("Columns in test: ", len(test_df.columns))

    X_train = train_df.drop(FEATURE_COLUMNS_TO_DROP, axis=1, errors='ignore')
    X_test = test_df.drop(FEATURE_COLUMNS_TO_DROP, axis=1, errors='ignore')

    ground_truth_df = pd.read_csv(ground_truth_train_file)
    if ground_truth_df is not None:
        y_train = ground_truth_df[ground_truth_df['patient_id'].isin(train_ids)]
        y_test = ground_truth_df[ground_truth_df['patient_id'].isin(test_ids)].groupby('patient_id').first().reset_index()
    else:
        # maybe the other dfs contain the target columns
        y_train = train_df[TARGET_COLUMNS]
        y_test = test_df[TARGET_COLUMNS]

    if not os.path.exists(DEV_DATA_DIR):
        os.makedirs(DEV_DATA_DIR)

    # Split sensor data into train and test if needed
    if sensor_data is not None:
        X_train_sensor = sensor_data[sensor_data['patient_id'].isin(train_ids)]
        X_test_sensor = sensor_data[sensor_data['patient_id'].isin(test_ids)]

        X_test_sensor = X_test_sensor.merge(X_test[['patient_id', 'init_t', 'target_t']], on='patient_id', how='left')

        # Filter out rows where 'measure_days_from_diagnosis' is less than 'init_t' or greater than 'target_t'
        X_test_sensor = X_test_sensor[(X_test_sensor['measure_days_from_diagnosis'] >= X_test_sensor['init_t']) &
                                      (X_test_sensor['measure_days_from_diagnosis'] <= X_test_sensor['target_t'])]
        X_test_sensor = X_test_sensor.drop(['init_t', 'target_t'], axis=1)

        X_train_sensor.to_csv(os.path.join(DEV_DATA_DIR, f'X_train_SENSOR.csv'), index=False)
        X_test_sensor.to_csv(os.path.join(DEV_DATA_DIR, f'X_test_SENSOR.csv'), index=False)

    # Save the CSV files in the new directory
    X_train.to_csv(os.path.join(DEV_DATA_DIR, f'X_train{SPLIT_FILE_SUFFIX}.csv'), index=False)
    y_train.to_csv(os.path.join(DEV_DATA_DIR, f'Y_train.csv'), index=False)

    X_test.to_csv(os.path.join(DEV_DATA_DIR, f'X_test{SPLIT_FILE_SUFFIX}.csv'), index=False)
    y_test.to_csv(os.path.join(DEV_DATA_DIR, f'Y_test.csv'), index=False)

# %%
